console.clear();
const { Engine, Render, World, Bodies, Body, Events, Constraint } = Matter;
const bodyRadius = 5;
const ctx = elcanvas.getContext(`2d`);
const engine = Engine.create();
const render = Render.create({
  element: elwrap,
  engine,
  options: { width: 256, height: 256 }
});

// Blade
//
const blade = Bodies.rectangle(128 + 32, 128 + 0, 32, 16, {});
Events.on(engine, `afterUpdate`, e => {
  Body.setAngularVelocity(blade, -0.1);
});
Events.on(engine, `collisionStart`, e => {
  for (const pair of e.pairs) {
    let body = null; 
    if (pair.bodyA == blade) {
      body = pair.bodyB;
    }
    if (pair.bodyB == blade) {
      body = pair.bodyA;
    }
    if (body) {
      Body.scale(body, 0.5, 0.5);
      body.customData.isCut = true;
      if (body.circleRadius < 1) {
        renew(body);
      }
    }
  }
});
World.add(engine.world, blade);

// Hang the blade on wall
//
const bladeConstraint = Constraint.create({
  bodyA: blade,
  pointB: { x: 128, y: 128 },
  length: 8,
  damping: 0
});
World.add(engine.world, bladeConstraint);

// Bodies
//
for (const i of Array(16).keys()) {
  const body = createBody(128, 0 - i * bodyRadius);
  World.add(engine.world, body);
}

// Ground
//
const ground = Bodies.rectangle(0, 256 + 64 / 2, 1e5, 64, { isStatic: true });
World.add(engine.world, ground);
Events.on(engine, `collisionStart`, e => {
  for (const pair of e.pairs) {
    if (pair.bodyA == ground) {
      renew(pair.bodyB);
    }
    if (pair.bodyB == ground) {
      renew(pair.bodyA);
    }
  }
});

// Render
//
Events.on(engine, `tick`, e => {
  ctx.globalCompositeOperation = `source-over`;
  ctx.fillStyle = `hsla(290,30%,80%,0.01)`;
  ctx.fillRect(0,0,256,256);
 
  for (const body of Matter.Composite.allBodies(engine.world)) {
    if (body === ground || body === blade) {
      continue;
    }
    ctx.fillStyle = body.customData.isCut
      ? body.customData.color
      : body.customData.initColor;
    ctx.beginPath();
    ctx.ellipse(
      body.position.x,
      body.position.y,
      16 * body.area / 48 * 2,
      256 * body.area / 48 * 2,
      body.angle,
      0,
      Math.PI * 2
    );
    ctx.globalCompositeOperation = `xor`; 
    // xor: made transparent where both overlap
    //      and drawn normal everywhere else.    
    // e.g. water spread fx
    ctx.fill();
  }
});

Engine.run(engine);
//Render.run(render);

function renew(body) {
  World.remove(engine.world, body);
  World.add(engine.world, createBody(128, 0));
}

function createBody(x, y) {
  const body = Bodies.circle(x, y, bodyRadius, {
    restitution: 1,
    density: 0.001,
    angle: -Math.PI / 16 + Math.random() * Math.PI / 8,
    frictionAir: 0.001,
    collisionFilter: {
      group: -1
    }
  });
  body.customData = {
    initColor: `hsla(0,0%,100%,0.01)`,
    color: `hsla(200,20%,45%,0.1)`,
    isCut: false
  };
  return body;
}